*setup
cap log close
log using 01_create_data.txt, text replace

***create data for CPS illustrations
use cpsmar10.dta, clear

*generate variables
*log wages
gen log_ern=log(ern_val) if ern_srce==1

gen tmp=(dis_hp==1)
bysort h_seq: egen disab_pr=max(tmp)
drop tmp

*race categories
gen rc_cat=1 if prdtrace==1
replace rc_cat=2 if prdtrace==2
replace rc_cat=3 if prdtrace==4
replace rc_cat=4 if prdtrace==3 | prdtrace>=5

*dummy if black
gen black=(prdtrace==2)

*marital status
gen married=(a_maritl<=3)

*part time
gen pttime=(a_wkstat>=3 & a_wkstat<=5)

*female
gen female=(a_sex==2)

*age categories
gen age_2544=(a_age>=25 & a_age<45)
gen age_4564=(a_age>=45 & a_age<65)
gen age_ge65=(a_age>=65)

*detailed age
gen d_agecat=(a_age<=24)
replace d_agecat=2 if a_age>=25 & a_age<35
replace d_agecat=3 if a_age>=35 & a_age<55
replace d_agecat=4 if a_age>=55 & a_age<65
replace d_agecat=5 if a_age>=65

*education categories
gen educ_cat=(a_hga<39)
replace educ_cat=2 if a_hga==39
replace educ_cat=3 if a_hga>39

*region (omitting south)
gen reg_ne=(gereg==1)
gen reg_mw=(gereg==2)
gen reg_w=(gereg==4)

*union member
gen union=(a_unmem==1)

*foreign born
gen for_born=(prcitshp>=3)

*non-profit
gen nprfit=(peio1cow==5)

*any farm income
gen farm_inc=(hfrval!=0)

*potential experience
gen pot_exp=a_age-6
replace pot_exp=pot_exp-4 if a_hga==32
replace pot_exp=pot_exp-6 if a_hga==33
replace pot_exp=pot_exp-8 if a_hga==34
replace pot_exp=pot_exp-9 if a_hga==35
replace pot_exp=pot_exp-10 if a_hga==36
replace pot_exp=pot_exp-11 if a_hga==37
replace pot_exp=pot_exp-12 if a_hga==38 | a_hga==39
replace pot_exp=pot_exp-14 if a_hga==40 | a_hga==41 | a_hga==42
replace pot_exp=pot_exp-16 if a_hga==43
replace pot_exp=pot_exp-18 if a_hga==44
replace pot_exp=pot_exp-19 if a_hga==45
replace pot_exp=pot_exp-21 if a_hga==46
replace pot_exp=min(a_age-16,pot_exp)
gen pot_exp_2=pot_exp^2
gen pot_exp_3=pot_exp^3
gen pot_exp_4=pot_exp^4

compress
keep h_seq female wexp a_clswkr married mjocc d_agecat a_famrel farm_inc a_hga gediv gtcbsasz rc_cat pehspnon a_mjind disab_pr union for_born pttime nprfit hrswk hunder18 pot_exp pot_exp_2 pot_exp_3 pot_exp_4 black gereg reg_ne reg_mw reg_w educ_cat age_* a_age a_enrlw h_type ern_val a_clswkr log_ern marsupwt i_ernval htotval
save cps_illustrations.dta, replace

*create data for ACS 2010 FS applications
use cpsmar10.dta, clear

*drop group quarters
drop if h_type==9
*create variable for which every hh member is needed
*presence of elderly person
gen tmp=(a_age>=60)
bysort h_seq: egen eldrly_pr=max(tmp)
drop tmp

gen tmp=(dis_hp==1)
bysort h_seq: egen disab_pr=max(tmp)
drop tmp

*income from capital dummy
*first def makes more sense to me, but includes 60% of all hh, ACS seems to be closer to second def
*gen tmp=(int_val>0 | div_val>0 | rnt_val>0)
gen tmp=(div_val>0 | rnt_val!=0)
bysort h_seq: egen cap_inc=max(tmp)
drop tmp

*keep only hh heads
keep if perrp<3

*whether elderly or disabled person present
gen eld_dis=(eldrly_pr==1 | disab_pr==1)

*create variables
gen female=(a_sex==2)
gen non_cit=(prcitshp==5)
gen r18=(hunder18>0)

*head hispanic
gen hisp=(pehspnon==1)

*recode variables
recode hfoodsp (2=0)

*rename variables to ACS names
ren hfoodsp fs
ren hfdval fs_amnt
ren h_numper np
ren hunder18 nc
ren frelu18 nrc
ren h_year year
ren hsup_wgt wgtp
ren a_age agep
ren htotval hincp
ren hearnval inc_earn
ren h_seq serialno
ren i_hfoods ffsp
ren i_hfdval ffsp_amnt

*get hh poverty line
gen hhpovtyp=1 if np==1 & agep<60
replace hhpovtyp=2 if np==1 & agep>=60
replace hhpovtyp=3 if np==2 & agep<60 & nrc==0
replace hhpovtyp=4 if np==2 & agep<60 & nrc==1
replace hhpovtyp=5 if np==2 & agep>=60 & nrc==0
replace hhpovtyp=6 if np==2 & agep>=60 & nrc==1
replace hhpovtyp=7 if np==3 & nrc==0
replace hhpovtyp=8 if np==3 & nrc==1
replace hhpovtyp=9 if np==3 & nrc==2
replace hhpovtyp=10 if np==4 & nrc==0
replace hhpovtyp=11 if np==4 & nrc==1
replace hhpovtyp=12 if np==4 & nrc==2
replace hhpovtyp=13 if np==4 & nrc==3
replace hhpovtyp=14 if np==5 & nrc==0
replace hhpovtyp=15 if np==5 & nrc==1
replace hhpovtyp=16 if np==5 & nrc==2
replace hhpovtyp=17 if np==5 & nrc==3
replace hhpovtyp=18 if np==5 & nrc==4
replace hhpovtyp=19 if np==6 & nrc==0
replace hhpovtyp=20 if np==6 & nrc==1
replace hhpovtyp=21 if np==6 & nrc==2
replace hhpovtyp=22 if np==6 & nrc==3
replace hhpovtyp=23 if np==6 & nrc==4
replace hhpovtyp=24 if np==6 & nrc==5
replace hhpovtyp=25 if np==7 & nrc==0
replace hhpovtyp=26 if np==7 & nrc==1
replace hhpovtyp=27 if np==7 & nrc==2
replace hhpovtyp=28 if np==7 & nrc==3
replace hhpovtyp=29 if np==7 & nrc==4
replace hhpovtyp=30 if np==7 & nrc==5
replace hhpovtyp=31 if np==7 & nrc==6
replace hhpovtyp=32 if np==8 & nrc==0
replace hhpovtyp=33 if np==8 & nrc==1
replace hhpovtyp=34 if np==8 & nrc==2
replace hhpovtyp=35 if np==8 & nrc==3
replace hhpovtyp=36 if np==8 & nrc==4
replace hhpovtyp=37 if np==8 & nrc==5
replace hhpovtyp=38 if np==8 & nrc==6
replace hhpovtyp=39 if np==8 & nrc==7
replace hhpovtyp=40 if np>=9 & nrc==0
replace hhpovtyp=41 if np>=9 & nrc==1
replace hhpovtyp=42 if np>=9 & nrc==2
replace hhpovtyp=43 if np>=9 & nrc==3
replace hhpovtyp=44 if np>=9 & nrc==4
replace hhpovtyp=45 if np>=9 & nrc==5
replace hhpovtyp=46 if np>=9 & nrc==6
replace hhpovtyp=47 if np>=9 & nrc==7
replace hhpovtyp=48 if np>=9 & nrc>=8

*merge poverty line
merge m:1 hhpovtyp year using povlines.dta

tab _merge
drop if _merge==2

*generate povery dummies
gen pov_head=(ftotval<=fpovcut)

*labels (only needed for vars not in ACS)
label var ffsp_amnt "Food Stamp amount allocated (CPS only)"

*keep only required variables
keep eldrly_pr disab_pr cap_inc eld_dis female r18 non_cit hisp fs_amnt np fs serialno wgtp agep hincp nc inc_earn pov_head ffsp ffsp_amnt
*save
compress
save cps10_extract.dta, replace

***ACS
use serialno st dis relp intp pernp agep sex cit hisp povpip using acs2010_population_us.dta, clear

*drop people from Puerto Rico
drop if st==72

*create variables for which every hh member is needed
bysort serialno: egen disab_pr=min(dis)
recode disab_pr (2=0)

*dummy if anyone in hh had capital income
gen tmp=(intp!=0 & intp!=1 & intp!=.)
bysort serialno: egen cap_inc=max(tmp)
drop tmp

*sum earned income over hh
bysort serialno: egen inc_earn=total(pernp)

*count children
gen tmp=(agep<18)
bysort serialno: egen nc=total(tmp)
drop tmp

*keep only hh heads
keep if relp==0

*create variables
gen female=(sex==2)
*head not a citizen
gen non_cit=(cit==5)

*head hispanic
recode hisp (1=0) (2/24=1)

*get household file
merge 1:1 serialno using acs2010_housing_us.dta, keepusing(fs ffsp r18 type r60 hincp np wgtp nrc)

*drop group quarters
drop if type==2 | type==3

*_merge==2 are empty units and group quarters
tab _merge
drop if _merge==2
drop _merge

*create variables in housing file

*dummy for fs receipt and amount
recode fs (2=0)
gen fs_amnt=.

*dummy whether elderly person present
gen eldrly_pr=(r60>0)

*whether elderly or disabled person present
gen eld_dis=(eldrly_pr==1 | disab_pr==1)

*get HH poverty line
*create HH types that determine poverty line -> using np as # of persons, nrc for number of children
gen hhpovtyp=1 if np==1 & agep<60
replace hhpovtyp=2 if np==1 & agep>=60
replace hhpovtyp=3 if np==2 & agep<60 & nrc==0
replace hhpovtyp=4 if np==2 & agep<60 & nrc==1
replace hhpovtyp=5 if np==2 & agep>=60 & nrc==0
replace hhpovtyp=6 if np==2 & agep>=60 & nrc==1
replace hhpovtyp=7 if np==3 & nrc==0
replace hhpovtyp=8 if np==3 & nrc==1
replace hhpovtyp=9 if np==3 & nrc==2
replace hhpovtyp=10 if np==4 & nrc==0
replace hhpovtyp=11 if np==4 & nrc==1
replace hhpovtyp=12 if np==4 & nrc==2
replace hhpovtyp=13 if np==4 & nrc==3
replace hhpovtyp=14 if np==5 & nrc==0
replace hhpovtyp=15 if np==5 & nrc==1
replace hhpovtyp=16 if np==5 & nrc==2
replace hhpovtyp=17 if np==5 & nrc==3
replace hhpovtyp=18 if np==5 & nrc==4
replace hhpovtyp=19 if np==6 & nrc==0
replace hhpovtyp=20 if np==6 & nrc==1
replace hhpovtyp=21 if np==6 & nrc==2
replace hhpovtyp=22 if np==6 & nrc==3
replace hhpovtyp=23 if np==6 & nrc==4
replace hhpovtyp=24 if np==6 & nrc==5
replace hhpovtyp=25 if np==7 & nrc==0
replace hhpovtyp=26 if np==7 & nrc==1
replace hhpovtyp=27 if np==7 & nrc==2
replace hhpovtyp=28 if np==7 & nrc==3
replace hhpovtyp=29 if np==7 & nrc==4
replace hhpovtyp=30 if np==7 & nrc==5
replace hhpovtyp=31 if np==7 & nrc==6
replace hhpovtyp=32 if np==8 & nrc==0
replace hhpovtyp=33 if np==8 & nrc==1
replace hhpovtyp=34 if np==8 & nrc==2
replace hhpovtyp=35 if np==8 & nrc==3
replace hhpovtyp=36 if np==8 & nrc==4
replace hhpovtyp=37 if np==8 & nrc==5
replace hhpovtyp=38 if np==8 & nrc==6
replace hhpovtyp=39 if np==8 & nrc==7
replace hhpovtyp=40 if np>=9 & nrc==0
replace hhpovtyp=41 if np>=9 & nrc==1
replace hhpovtyp=42 if np>=9 & nrc==2
replace hhpovtyp=43 if np>=9 & nrc==3
replace hhpovtyp=44 if np>=9 & nrc==4
replace hhpovtyp=45 if np>=9 & nrc==5
replace hhpovtyp=46 if np>=9 & nrc==6
replace hhpovtyp=47 if np>=9 & nrc==7
replace hhpovtyp=48 if np>=9 & nrc>=8


*merge poverty line
gen year=2010
merge m:1 hhpovtyp year using povlines.dta
drop if _merge==2
drop _merge

*income relative to pov line for hh
gen incpov=hincp/pov_ln*100

*generate povery dummies
gen pov_hh=(hincp<=pov_ln)
gen pov_head=(povpip<=100)

*get food stamp rules
merge m:1 np year using fs_rules.dta
*note: no hh with 18 persons in 2010, so some values do not merge
drop if _merge==2
drop _merge
replace st_deduc=st_deduc_ak if st==2
replace st_deduc=st_deduc_hi if st==15
replace max_shltr=max_shltr_ak if st==2
replace max_shltr=max_shltr_hi if st==15

*label variables
label var fs "Yearly Food Stamp/SNAP recipiency"
label var fs_amnt "Food stamp amount (yearly amount)"
label var eldrly_pr "Person 60 or over lives in HH"
label var disab_pr "Person with disability lives in HH"
label var female "HH head is female"
label var cap_inc "HH has any capital income"
label var inc_earn "Earned Income, HH total"
label var non_cit "HH head is not a US citizen"
label var nc "Number of Children in HH, sample count"
label var max_amnt "Maximum FS amount"
label var st_deduc "FS standard deduction for HH"
label var max_shltr "FS maximum shelter deduction for HH"
label var hisp "HH head hispanic"
label var pov_head "HH head in family below poverty line"
label var eld_dis "Elderly or disabled person present in HH"
label var ffsp "Food Stamp amount (ACS until 2007) or receipt (after 2007,CPS) imputed"

*define value labels
label define noyes 0 "No" 1 "Yes"

*attach value labels
label values fs eldrly_pr disab_pr female cap_inc non_cit hisp pov_head eld_dis noyes

*keep only variables needed for analysis
keep serialno disab_pr cap_inc inc_earn nc female non_cit hisp agep pov_head fs ffsp fs_amnt wgtp hincp np eldrly_pr eld_dis r18 max_shltr st_deduc max_amnt
*save
gen acs=1
append using cps10_extract.dta
replace acs=0 if acs==.
label var acs "Source of observation is the ACS"
label def acs 0 "CPS" 1 "ACS"
compress
save pooled_data.dta, replace

***Create ACS 2010 person level file for application 3

*restrict sample: work hours not missing, positive earnings, 18 and older, not in school or army, not working without pay or not at all, work hours not imputed, worked 50 weeks or more (+throwing out GQ below)
use serialno st sex rac1p mar pernp wkhp pwgtp agep schl schg occp cow fwkhp wkw if wkhp!=. & pernp>0 & agep>=18 & schg==. & occp<9800 & cow>=1 & cow<=7 & fwkhp==0 & wkw==1 using acs2010_population_us.dta, clear

*drop people from Puerto Rico and vars not needed
drop if st==72
drop schg occp cow fwkhp wkw

*create covariates for regression
gen female=(sex==2)

gen black=(rac1p==2)

gen married=(mar==1)

*education categories
gen educ_lths=(schl<=15)
gen educ_hs=(schl==16 | schl==17)
gen educ_soco=(schl>17 & schl<21)
gen educ_cmor=(schl>=21 & schl!=.)

*potential experience
gen pot_exp=agep-6
replace pot_exp=pot_exp-schl+3 if schl>=4 & schl<=14
replace pot_exp=pot_exp-12 if schl>14 & schl<=18
replace pot_exp=pot_exp-14 if schl==19 | schl==20
replace pot_exp=pot_exp-16 if schl==21
replace pot_exp=pot_exp-18 if schl==22
replace pot_exp=pot_exp-19 if schl==23
replace pot_exp=pot_exp-21 if schl==24
gen pot_exp_2=pot_exp^2
gen pot_exp_3=pot_exp^3
gen pot_exp_4=pot_exp^4

*merge required variables from housing record
merge m:1 serialno using acs2010_housing_us.dta, keepusing(region type np nrc)
drop if type>1
keep if _merge==3

*region dummies
gen reg_ne=(region==1)
gen reg_mw=(region==2)
gen reg_s=(region==3)
gen reg_w=(region==4)

keep serialno pernp wkhp pwgtp female black married educ_* reg_* pot_exp* nrc np
compress
save a3_acs_extract.dta, replace
