*setup
capture log close
log using 02_illustrations.txt, text replace

*set locals

*Illustration 1: define match/partial match/non-match vars
local i1mvar "i.female i.wexp i.a_clswkr i.married i.mjocc i.d_agecat i.a_famrel i.farm_inc"
local i1imvar "i.a_hga i.gediv i.gtcbsasz i.rc_cat"
local i1nmvar "i.pehspnon i.a_mjind i.disab_pr i.union i.for_born i.pttime i.nprfit hrswk hunder18 pot_exp pot_exp_2 pot_exp_3 pot_exp_4"

*Illustration 2
*variables for hot deck
local i2cellvar black married pttime female gereg
*covariates (excluding imputed variable)
local i2covar "pot_expi pot_exp_2i pot_exp_3i pot_exp_4i black married pttime reg_ne reg_mw reg_w female"
*# of iterations (defaults to 10 if unset)
local iter=1000
*# and names of education categories
local ned=15
local educ `""1st-4th grade" "5-6th grade" "7-8th grade" "9th grade" "10th grade" "11th grade" "12th, no diploma" "HS graduate" "Some college, no degree" "Non-academic Assoc Degr" "Academic Assoc Degr" "Bachelor's degree" "Master's degree" "Professional School degr" "Doctorate Degree""'

*Illustration 3
*cell variables for hot deck/conditioning variables for cond mean
local i3cellvar black female age_* educ_cat


***Illustration 1: effect of variables that are not in hot deck
*setup
use cps_illustrations.dta, clear

*restrict sample (18 and older, no students, army or group quarters, pos earnings, no (emp w/o pay, never worked, NIU (incl children and armed forces))
keep if a_age>=18 & a_enrlw!=1 & mjocc!=11 & h_type!=9 & ern_val>0 & a_clswkr>0 & a_clswkr<7

*run regressions and tests vs. 0
reg log_ern `i1mvar' `i1imvar' `i1nmvar' [pw=marsupwt]
est store all
*test whether sets of variables are jointly zero
*test all match vars
qui {
testparm `i1mvar' `i1imvar'
mat def test0=(r(p),r(F),r(df))
testparm `i1mvar'
mat def test0=(nullmat(test0)\(r(p),r(F),r(df)))
testparm `i1imvar'
mat def test0=(nullmat(test0)\(r(p),r(F),r(df)))
*test all non-match vars
testparm `i1nmvar' 
mat def test0=(nullmat(test0)\(r(p),r(F),r(df)))
*test potential experience jointly
testparm pot_exp*
mat def test0=(nullmat(test0)\(r(p),r(F),r(df)))
local rnames `""all_match_var" "comp_match_var" "incomp_match_var" "non-match_var" "pot_exp_var""'
*test sets of dummies jointly
foreach x in `i1mvar' `i1imvar' `i1nmvar' {
	if strmatch(substr("`x'",1,2),"i.")==1 {
		testparm `x'
		mat def test0=(nullmat(test0)\(r(p),r(F),r(df)))
		local rnames=`"`rnames' "'+substr("`x'",3,.)
	}
}
matrix rownames test0=`rnames'
}

reg log_ern `i1mvar' `i1imvar' `i1nmvar' if i_ernval==0 [pw=marsupwt]
est store reported
*test whether sets of variables are jointly zero
*test all match vars
qui {
testparm `i1mvar' `i1imvar'
mat def test=(r(p),r(F),r(df))
testparm `i1mvar'
mat def test=(test\(r(p),r(F),r(df)))
testparm `i1imvar'
mat def test=(test\(r(p),r(F),r(df)))
*test all non-match vars
testparm `i1nmvar' 
mat def test=(test\(r(p),r(F),r(df)))
*test potential experience jointly
testparm pot_exp*
mat def test=(test\(r(p),r(F),r(df)))
*test sets of dummies jointly
foreach x in `i1mvar' `i1imvar' `i1nmvar' {
	if strmatch(substr("`x'",1,2),"i.")==1 {
		testparm `x'
		mat def test=(test\(r(p),r(F),r(df)))
	}
}
mat def test0=(test0,test)
}

reg log_ern `i1mvar' `i1imvar' `i1nmvar' if i_ernval==1 [pw=marsupwt]
est store imputed
local a: colnames e(b)
qui {
testparm `i1mvar' `i1imvar'
mat def test=(r(p),r(F),r(df))
testparm `i1mvar'
mat def test=(test\(r(p),r(F),r(df)))
testparm `i1imvar'
mat def test=(test\(r(p),r(F),r(df)))
*test all non-match vars
testparm `i1nmvar' 
mat def test=(test\(r(p),r(F),r(df)))
*test potential experience jointly
testparm pot_exp*
mat def test=(test\(r(p),r(F),r(df)))
*test sets of dummies jointly
foreach x in `i1mvar' `i1imvar' `i1nmvar' {
	if strmatch(substr("`x'",1,2),"i.")==1 {
		testparm `x'
		mat def test=(test\(r(p),r(F),r(df)))
	}
}
mat def test0=(test0,test)
}

matrix colnames test0="all_p-value" "all_F-Stat" "all_df" "reported_p-value" "reported_F-Stat" "reported_df" "imputed_p-value" "imputed_F-Stat" "imputed_df"

*tables
est table all reported imputed, stats(N r2) se
est table all reported imputed, stats(N r2) star

*export results
outreg2 [all reported imputed] using i1_reg.xml, e(N r2) se excel replace

*decompose varlist for tests
local i=1
local cmvar ""
tokenize "`a'"
foreach x in `i1mvar' {
	local y=subinstr("`x'","i.","",1)
	if strmatch("``i''","*`y'")==1 {
		if strmatch("``i''","*b.*")==1 local i=`i'+1
		*variable list for tests
		local cn_`y' "``i''"
		local i=`i'+1
		while strmatch("``i''","*`y'")==1 {
			if strmatch("``i''","9730*")==0 & strmatch("``i''","1710*")==0 & strmatch("``i''","ib.")==0 local cn_`y' "`cn_`y'' ``i''"
			local i=`i'+1
		}
	}
	else {
		di as error "Something went wrong, expected `y', got ``i''"
		error
	}
*get full varlist for tests
	local cmvar "`cmvar' `cn_`y''"
}

local cimvar ""
foreach x in `i1imvar' {
	local y=subinstr("`x'","i.","",1)
	if strmatch("``i''","*`y'")==1 {
		if strmatch("``i''","*b.*")==1 local i=`i'+1
		*variable list for tests
		local cn_`y' "``i''"
		local i=`i'+1
		while strmatch("``i''","*`y'")==1 {
			if strmatch("``i''","ib.")==0 local cn_`y' "`cn_`y'' ``i''"
			local i=`i'+1
		}
	}
	else {
		di as error "Something went wrong, expected `y', got ``i''"
		error
	}
*get full varlist for tests
	local cimvar "`cimvar' `cn_`y''"
}

local cnmvar ""
foreach x in `i1nmvar' {
	local y=subinstr("`x'","i.","",1)
	if strmatch("``i''","*`y'")==1 {
		if strmatch("``i''","*b.*")==1 local i=`i'+1
		*variable list for tests
		local cn_`y' "``i''"
		local i=`i'+1
		while strmatch("``i''","*`y'")==1 {
			if strmatch("``i''","ib.")==0 local cn_`y' "`cn_`y'' ``i''"
			local i=`i'+1
		}
	}
	else {
		di as error "Something went wrong, expected `y', got ``i''"
		error
	}
*get full varlist for tests
	local cnmvar "`cnmvar' `cn_`y''"
}

*tests of equality
qui {
	reg log_ern `i1mvar' `i1imvar' `i1nmvar' if i_ernval==0 [iw=marsupwt]
	est store reported
	reg log_ern `i1mvar' `i1imvar' `i1nmvar' if i_ernval==1 [iw=marsupwt]
	est store imputed
}
suest reported imputed

*joint test that all coef are equal
test [reported_mean=imputed_mean], common
*store results
mat def testeq=(r(p),r(chi2),r(df))

*joint test that match variables are equal
test [reported_mean=imputed_mean]: `cmvar' `cimvar'
mat def testeq=(testeq\(r(p),r(chi2),r(df)))
test [reported_mean=imputed_mean]: `cmvar'
mat def testeq=(testeq\(r(p),r(chi2),r(df)))
test [reported_mean=imputed_mean]: `cimvar'
mat def testeq=(testeq\(r(p),r(chi2),r(df)))

*joint test that non-match variables are equal
test [reported_mean=imputed_mean]: `cnmvar'
mat def testeq=(testeq\(r(p),r(chi2),r(df)))
*test potential experience jointly
test [reported_mean=imputed_mean]: pot_exp pot_exp_2 pot_exp_3 pot_exp_4
mat def testeq=(testeq\(r(p),r(chi2),r(df)))

local rnameseq `""all_var" "all_match_var" "comp_match_var" "incomp_match_var" "non-match_var" "pot_exp_var""'

*joint test that groups of dummies are equal
foreach x in `i1mvar' `i1imvar' `i1nmvar' {
	if strmatch("`x'","*.*") {
		local y=subinstr("`x'","i.","",1)
		test [reported_mean=imputed_mean]: `cn_`y''
		mat def testeq=(testeq\(r(p),r(chi2),r(df)))
		local rnameseq `"`rnameseq' `y'"'
	}
}	

*testing variable by variable
foreach var in `cmvar' `cimvar' `cnmvar' {
	test [reported_mean=imputed_mean]:`var'
	*store result
	mat def testeq=(testeq\(r(p),r(chi2),r(df)))
	local x=subinstr("`var'",".","",.)
	local rnameseq `"`rnameseq' `x'"'
}
matrix colnames testeq="p-value" "chi2-Stat" "df"
matrix rownames testeq=`rnameseq'

**compute gaps/deviations for dummy variables
*all dummies
capture mat drop alld
local rnalld ""
foreach x in `cmvar' `cimvar' `cnmvar' {
	if strmatch("`x'","*.*") {
		capture noisily mat def tmp=[reported_mean]_b[`x']
		if _rc==0 capture noisily mat def tmp=(tmp,[imputed_mean]_b[`x'])
		if _rc==0 & tmp[1,1]!=0 & tmp[1,2]!=0 mat def alld=(nullmat(alld)\tmp)
		local y=subinstr("`x'",".","",.)
		local rnalld "`rnalld' `y'"
	}
}
matrix rownames alld=`rnalld'
matrix colnames alld=reported imputed
*compute deviations
mata: st_matrix("dev",colsum(abs(st_matrix("alld"))):/rows(st_matrix("alld")))
matrix colnames dev=reported imputed

*match dummies
*all
capture mat drop amatchd
local rnamatchd ""
foreach x in `cmvar' `cimvar' {
	if strmatch("`x'","*.*") {
		capture noisily mat def tmp=[reported_mean]_b[`x']
		if _rc==0 capture noisily mat def tmp=(tmp,[imputed_mean]_b[`x'])
		if _rc==0 & tmp[1,1]!=0 & tmp[1,2]!=0 mat def amatchd=(nullmat(amatchd)\tmp)
		local y=subinstr("`x'",".","",.)
		local rnamatchd "`rnamatchd' `y'"
	}
}
matrix rownames amatchd=`rnamatchd'
matrix colnames amatchd=reported imputed
*compute deviations
mata: st_matrix("dev",(st_matrix("dev")\colsum(abs(st_matrix("amatchd"))):/rows(st_matrix("amatchd"))))

*complete match
capture mat drop cmatchd
local rncmatchd ""
foreach x in `cmvar' {
	if strmatch("`x'","*.*") {
		capture noisily mat def tmp=[reported_mean]_b[`x']
		if _rc==0 capture noisily mat def tmp=(tmp,[imputed_mean]_b[`x'])
		if _rc==0 & tmp[1,1]!=0 & tmp[1,2]!=0 mat def cmatchd=(nullmat(cmatchd)\tmp)
		local y=subinstr("`x'",".","",.)
		local rncmatchd "`rncmatchd' `y'"
	}
}
matrix rownames cmatchd=`rncmatchd'
matrix colnames cmatchd=reported imputed
*compute deviations
mata: st_matrix("dev",(st_matrix("dev")\colsum(abs(st_matrix("cmatchd"))):/rows(st_matrix("cmatchd"))))

*incomplete match
capture mat drop imatchd
local rnimatchd ""
foreach x in `cimvar' {
	if strmatch("`x'","*.*") {
		capture noisily mat def tmp=[reported_mean]_b[`x']
		if _rc==0 capture noisily mat def tmp=(tmp,[imputed_mean]_b[`x'])
		if _rc==0 & tmp[1,1]!=0 & tmp[1,2]!=0 mat def imatchd=(nullmat(imatchd)\tmp)
		local y=subinstr("`x'",".","",.)
		local rnimatchd "`rnimatchd' `y'"
	}
}
matrix rownames imatchd=`rnimatchd'
matrix colnames imatchd=reported imputed
*compute deviations
mata: st_matrix("dev",(st_matrix("dev")\colsum(abs(st_matrix("imatchd"))):/rows(st_matrix("imatchd"))))

*non-match dummies
capture mat drop nmatchd
local rndnm ""
foreach x in `cnmvar' {
	if strmatch("`x'","*.*") {
		capture noisily mat def tmp=[reported_mean]_b[`x']
		if _rc==0 capture noisily mat def tmp=(tmp,[imputed_mean]_b[`x'])
		if _rc==0 & tmp[1,1]!=0 & tmp[1,2]!=0 mat def nmatchd=(nullmat(nmatchd)\tmp)
		local y=subinstr("`x'",".","",.)
		local rndnm "`rndnm' `y'"
	}

}
matrix rownames nmatchd=`rndnm'
matrix colnames nmatchd=reported imputed
*compute deviations
mata: st_matrix("dev",(st_matrix("dev")\colsum(abs(st_matrix("nmatchd"))):/rows(st_matrix("nmatchd"))))
local rndev `""all_dummies" "all_match_dummies" "comp_match_dummies" "incomp_match_dummies" "non_match_dummies""'

*deviations for each set of dummies
foreach var in `i1mvar' `i1imvar' `i1nmvar' {
	if strmatch("`var'","*.*") {
		local x=substr("`var'",3,.)
		capture matrix drop `x'd
		foreach y in `cn_`x'' {
			capture noisily mat def tmp=[reported_mean]_b[`y']
			if _rc==0 capture noisily mat def tmp=(tmp,[imputed_mean]_b[`y'])
			if _rc==0 & tmp[1,1]!=0 & tmp[1,2]!=0 mat def `x'd=(nullmat(`x'd)\tmp)
		}
		*compute deviations
		mata: st_matrix("dev",(st_matrix("dev")\colsum(abs(st_matrix("`x'd"))):/rows(st_matrix("`x'd"))))
		local rndev `"`rndev' `x'"'
		matrix drop `x'd
	}
}
matrix rownames dev=`rndev'
matrix colnames dev=reported imputed

*organize and put tests into table
*convert to variables & label them
svmat test0
svmat testeq
svmat dev

*label variables and write rownames
foreach x in test0 testeq dev {
	local cname: colnames `x'
	tokenize "`cname'"
	local i=1
	while "``i''"!="" {
		label var `x'`i' "``i''"
		local i=`i'+1
	}
	local rname: rownames `x'
	local k`x'=rowsof(`x')
	tokenize "`rname'"
	gen n`x'=""
	local i=1
	while `i'<=`k`x'' {
		qui: replace n`x'="``i''" if _n==`i'
		local i=`i'+1
	}
}

*export
export excel ntest0 test0* using i1.xls if _n<=`ktest0', sh("sig test") replace firstrow(varl)
export excel ntesteq testeq* using i1.xls if _n<=`ktesteq', sh("eq test") sheetreplace firstrow(varl)
export excel ndev dev* using i1.xls if _n<=`kdev', sh("deviations") sheetreplace firstrow(varl)

*clean up
drop test0* testeq* dev* ndev ntest0 ntesteq
mat drop dev testeq test0 tmp imatchd cmatchd amatchd nmatchd alld testeq test0 test

***Illustration 2: Bias in coef and SEs when using imputed values
*illustrates effect on parameters and SEs

*generate cell variables for hot deck
egen cells=group(`i2cellvar')
set matsize 5000

*start loop
if "`iter'"=="" local iter=10
local k=wordcount("`i2covar'")+`ned'+2
mat def coef_t=J(`iter',`k',.)
mat def se_t=J(`iter',`k',.)
mat def coef=J(`iter',`k',.)
mat def se=J(`iter',`k',.)
local i=1
qui {
while `i'<=`iter' {
n: if mod(`i',10)==0 di "Iteration `i'"
*split sample in halves
gen imp=(runiform()<0.5)

*generate variable to be imputed
gen educ_imp=a_hga if imp==0

*impute
mi set wide
mi register imputed educ_imp
mi impute pmm educ_imp [pw=marsupwt], by(cells) add(1)

*generate potential experience
gen pot_expi=a_age-6
replace pot_expi=pot_expi-4 if _1_educ_imp==32
replace pot_expi=pot_expi-6 if _1_educ_imp==33
replace pot_expi=pot_expi-8 if _1_educ_imp==34
replace pot_expi=pot_expi-9 if _1_educ_imp==35
replace pot_expi=pot_expi-10 if _1_educ_imp==36
replace pot_expi=pot_expi-11 if _1_educ_imp==37
replace pot_expi=pot_expi-12 if _1_educ_imp==38 | _1_educ_imp==39
replace pot_expi=pot_expi-14 if _1_educ_imp==40 | _1_educ_imp==41 | _1_educ_imp==42
replace pot_expi=pot_expi-16 if _1_educ_imp==43
replace pot_expi=pot_expi-18 if _1_educ_imp==44
replace pot_expi=pot_expi-19 if _1_educ_imp==45
replace pot_expi=pot_expi-21 if _1_educ_imp==46
replace pot_expi=min(a_age-16,pot_expi)
gen pot_exp_2i=pot_expi^2
gen pot_exp_3i=pot_expi^3
gen pot_exp_4i=pot_expi^4

*compute true and naive SEs and show how they differ
*"true" half
reg log_ern i.a_hga `i2covar' if imp==1 [pw=marsupwt], robust
mat def coef_t[`i',1]=e(b)
mat def se_t[`i',1]=vecdiag(e(V))

*imputed half
reg log_ern i._1_educ_imp `i2covar' if imp==1 [pw=marsupwt]
mat def coef[`i',1]=e(b)
mat def se[`i',1]=vecdiag(e(V))

*clean up
mi unset
drop imp educ_imp educ_imp_1_ mi_miss pot_expi pot_exp_2i pot_exp_3i pot_exp_4i
local i=`i'+1
}
}
*compute biases in coefficients
mat def coef_tm=coef_t[1...,2...]'*J(`iter',1,1/`iter')
matrix rownames coef_tm=`educ' `i2covar' cons
mat def coef_m=coef[1...,2...]'*J(`iter',1,1/`iter')
matrix rownames coef_m=`educ' `i2covar' cons
mat def coef_d=coef_m-coef_tm
matrix rownames coef_d=`educ' `i2covar' cons

*compute SE of true coef (asympt: mean of se_t, bs: variance of coef_t), true SEs (variance of coef) and naive se (mean of se) of coefficients from imputed obs
mata: st_matrix("se_tm",sqrt(st_matrix("se_t"))[|1,2\.,.|]'*J(`iter',1,1/`iter'))
matrix rownames se_tm=`educ' `i2covar' cons
mata: st_matrix("se_tbs",sqrt(diagonal(variance(st_matrix("coef_t")[|1,2\.,.|]))))
matrix rownames se_tbs=`educ' `i2covar' cons
mata: st_matrix("se_m",sqrt(st_matrix("se"))[|1,2\.,.|]'*J(`iter',1,1/`iter'))
matrix rownames se_m=`educ' `i2covar' cons
mata: st_matrix("se_bs",sqrt(diagonal(variance(st_matrix("coef")[|1,2\.,.|]))))
matrix rownames se_bs=`educ' `i2covar' cons
mat def se_di=se_m-se_bs
matrix rownames se_di=`educ' `i2covar' cons
mat def se_dni=se_tm-se_tbs
matrix rownames se_dni=`educ' `i2covar' cons
mat def se_dbs=se_bs-se_tbs
matrix rownames se_dbs=`educ' `i2covar' cons


*Results
*average coefficients using non-imputed sample
mat list coef_tm
*average coefficients using imputed sample
mat list coef_m
*average analytic variance of non-imputed sample
mat list se_tm
*bootstrapped variance of non-imputed sample
mat list se_tbs
*(biased) average analytic variance of imputed sample
mat list se_m
*bootstrapped variance of imputed sample
mat list se_bs

*bias in imputed coef (imp-true)
mat list coef_d
*bias in imputed SEs (analytic-bootstrapped)
mat list se_di
*difference analytic-bootstrapped SEs for non-imp
mat list se_dni
*efficiency loss from imputation (imp bs - non-imp bs SEs)
mat list se_dbs

*export resuts to table
mat def res=(coef_tm, coef_m, coef_d, se_tm, se_tbs, se_dni, se_m, se_bs, se_di, se_dbs)
mat colnames res="true coef" "imp coef" "bias coef" "true asy SE" "true bs SE" "true asy-bs SE" "imp asy SE" "imp bs SE" "bias imp SE (asy-bs)" "imp bs-true bs SE"
svmat res
*label variables
label var res1 "Average Non-Imp Coefficients"
label var res2 "Average Imputed Coefficients"
label var res3 "Bias Coefficients"
label var res4 "Non-Imp Asymptotic SEs"
label var res5 "Non-Imp Bootstr SEs"
label var res6 "Non-Imp Asympt.-Bootstr SEs"
label var res7 "Imputed Asymptotic SEs"
label var res8 "Imputed Bootstr SEs"
label var res9 "Bias Imputed SEs (Asympt.-Bootstr)"
label var res10 "Efficiency loss (Imp bs - non-imp bs SEs)"


local k=rowsof(res)
gen x=_n if _n<=`k'
local i=1
while `i'<=`k' {
	mat def a=res[`i',1...]
	local a: rownames a
	label define xlab `i' "`a'", modify
	local i=`i'+1
}
label values x xlab

export excel x res* using i2.xls if _n<=`k', replace firstrow(varl)

*graph difference in returns to educ
svmat coef_tm, n(coef_tm)
svmat coef_m, n(coef_imp)
replace coef_tm1=. if x>`ned'
replace coef_imp1=. if x>`ned'

line coef_tm1 coef_imp1 x if _n<=`ned', xlabel(1(1)`ned',valuelabel angle(45)) title("Attenuation in Education Gradient") subtitle("Non-Imputed vs Imputed Observations") legend(label(1 "Observed") label(2 "Imputed")) ytitle("Log Earnings Gap") xtitle("") lpattern(solid dash)
graph save i2_educ, asis replace
graph export i2_educ.eps, replace

drop res* coef_tm* coef_imp* x cells
matrix drop coef_tm coef_m coef_d res se_dbs se_dni se_di se_bs se_m se_tbs se_tm coef se coef_t se_t a

***Illustration 3: Different goals require different techniques
*impute one half using conditional mean and hotdeck. income density only, see previous version for regressions

*split sample in halves
gen imp=(runiform()<0.5)
*cell variable for hot deck imputation
egen cells=group(`i3cellvar')

*conditional mean imputation
gen inc_cm=htotval if imp==0
reg inc_cm i.cells if imp==0
predict tmp
replace inc_cm=tmp if imp==1
drop tmp

*hot deck imputation
gen inc_hd=htotval if imp==0
mi set wide
mi register imputed inc_hd
mi impute pmm inc_hd [pw=marsupwt], by(cells) add(1)

*check marginal distributions
twoway kdensity htotval [aw=marsupwt] if imp==1, range(-10000 250000) lpattern(solid) lc(black) || kdensity inc_cm [aw=marsupwt] if imp==1, range(-10000 250000) lpattern(dash) || kdensity _1_inc_hd [aw=marsupwt] if imp==1, range(-10000 250000) lpattern(shortdash) lc(blue) legend(label(1 "Survey Response") label(2 "Conditional Mean Imputation") label(3 "Hot Deck Imputation")) title("Income Distribution, Different Imputation Methods") subtitle("2010 CPS ASEC") ytitle("Density") xtitle("Household Income")
graph save i3_dens, asis replace
graph export i3_dens.eps, replace

*K-S test of equality of distributions, true vs. hot deck
preserve
keep if imp==1
stack htotval marsupwt _1_inc_hd marsupwt, into(inc marsupwt) clear
ksmirnov2 inc [aw=marsupwt], by(_stack)
restore

*K-S test of equality of distributions, true vs. conditional mean
preserve
keep if imp==1
stack htotval marsupwt inc_cm marsupwt, into(inc marsupwt) clear
ksmirnov2 inc [aw=marsupwt], by(_stack)
restore

mi unset
drop imp cells inc_hd inc_cm inc_hd_1_ mi_miss
capture log close


