*general setup, locals, etc.

*define locals
*all variables I need after imputation: acs/orig indicator, fs amounts & imputations, weights, covariates for FS regressions, conditioning variables 
local all "orig acs wgtp fs_amnt fs_amnt_* np nc eldrly_pr disab_pr female np_* r18 inc_earn_* cap_inc pov_head hisp non_cit"

*covariates to predict amounts
local condvar "np_* nc r18 eldrly_pr disab_pr inc_earn_* cap_inc pov_head female hisp non_cit"
local pmmvar "np_* nc r18 eldrly_pr disab_pr inc_earn_* cap_inc pov_head female hisp non_cit"

*variables that form cells for classic HD
local hdcells "np2 nc2 eld_dis"

*variables to be used as covariates in Food Stamp regressions
local covar "np nc eldrly_pr disab_pr female"

*models for cond dist
local model ltrnorm

*restrict income-> set to high value (e.g. 2m) if no restriction should be imposed (otherwise if statements won't work)
local maxinc inc_earn<=200000

*variables for summary statistics
local sstat "`condvar' np fs_amnt fs eld_dis eldrly_pr disab_pr female"

**locals for application 3
*all variables needed
local all2 

*covariates to predict amounts
local condvar2 "female black married educ_hs educ_soco educ_cmor reg_ne reg_mw reg_w pot_exp pot_exp_2 pot_exp_3 pot_exp_4 nrc np"
local pmmvar2 "female black married educ_hs educ_soco educ_cmor reg_ne reg_mw reg_w pot_exp pot_exp_2 pot_exp_3 pot_exp_4 nrc np"

*variables that form cells for classic HD
local hdcells2 "female black married educ_hs educ_soco educ_cmor reg_ne reg_mw reg_w"

*variables to be used as additional covariates (besides wkhp)
local covar2 "female black married educ_hs educ_soco educ_cmor reg_ne reg_mw reg_w pot_exp pot_exp_2 pot_exp_3 pot_exp_4"

*models for cond dist
local model2 ltrnorm

*variables for summary statistics
local sstat2 "`condvar2' pernp wkhp wkhp_* nrc np"

*number of draws for simulation
local draw=20

*number of iterations for ck test
local iter 20

*list of variables for which higher order terms need to be created
local higher inc_earn

*number of higher order terms to include
local order 3

*local indicating whether whole file is running
local full=1

*log
cap log close
log using 03_analysis.txt, text replace

******Different ways of calculating FS amounts
***get data, setup
use _all if fs==1 & `maxinc' & agep>18 using pooled_data.dta, clear

****create additional variables
*recode np
replace np=7 if np>7
qui: tab np, gen(np_)
drop np_1

*np2 and nc2 for hot deck cells
gen np2=np
replace np2=5 if np2>5

gen nc2=nc
replace nc2=5 if nc2>5

*higher order terms of variables->creates first order term as well, so var_* includes all terms, var* will include linear term twice
foreach var of varlist `higher' {
	local i=1
	while `i'<=`order' {
		gen `var'_`i'=`var'^`i'
		local i=`i'+1
	}
}

***calculate fs amounts based on program rules
*calculate adjusted income
gen tmp=hincp/12-0.2*inc_earn/12-max_shltr-st_deduc if acs==1
replace tmp=0 if tmp<0
*subtract 0.3*adjusted income from max benefit amount to get amount
gen fs_amnt_rules=(max_amnt-0.3*tmp)*12 if acs==1 & ffsp==0
replace fs_amnt_rules=0 if (fs_amnt_rules<0 | fs_amnt_rules==.) & acs==1 & ffsp==0
drop tmp


***regression imputation
gen fs_amnt_reg=0 if acs==1 & ffsp==0
reg fs_amnt `condvar' [pw=wgtp] if acs==0 & ffsp_amnt==0
predict tmp if acs==1
replace fs_amnt_reg=tmp if acs==1 & ffsp==0
drop tmp


***predictive means match
*MI setup
mi set wide
gen fs_amnt_pmm=fs_amnt/np if acs==0 & ffsp_amnt==0
mi register imputed fs_amnt_pmm
*do imputations
mi impute pmm fs_amnt_pmm `pmmvar' if ffsp==0 [pw=wgtp],add(1)
replace fs_amnt_pmm=_1_fs_amnt_pmm if acs==1 & fs_amnt_pmm==. & ffsp==0
replace fs_amnt_pmm=fs_amnt_pmm*np
*get rid of mi settings
mi unset
drop fs_amnt_pmm_1_ mi_miss

***classic hot deck
mi set wide
gen fs_amnt_hd=fs_amnt/np if acs==0 & ffsp_amnt==0
mi register imputed fs_amnt_hd
*do imputations
mi impute pmm fs_amnt_hd if ffsp==0 [pw=wgtp],add(1) by(`hdcells')
replace fs_amnt_hd=_1_fs_amnt_hd if acs==1 & fs_amnt_hd==. & ffsp==0
replace fs_amnt_hd=fs_amnt_hd*np 
*get rid of mi settings
mi unset
drop fs_amnt_hd_1_ mi_miss

*save imputed data.
append using pooled_data.dta, gen(tmp)
keep if tmp==0 | (tmp==1 & fs==0 & agep>18 & `maxinc')
replace fs_amnt_rules=0 if tmp==1
replace fs_amnt_reg=0 if tmp==1
replace fs_amnt_pmm=0 if tmp==1
drop tmp
*save
compress
save pooled_imp.dta, replace

*******Conditional Density Method
*load data if `full'!=1, else drop non-recipients and ACS data (i.e. only non-imputed FS recipients in 2010 CPS).
if `full'==1 keep if acs==0 & fs==1 & ffsp_amnt==0 
else use _all if acs==0 & fs==1 & ffsp_amnt==0 using pooled_imp.dta, clear

*generate percentiles to evaluate conditional mean
pctile p_cps10=fs_amnt if acs==0 [pw=wgtp], nq(51)

***estimate cond dist in cps
*run program, do CM test, save graph and matrices
conddens fs_amnt `condvar' [pw=wgtp], model(`model') cmp(at(p_cps10) noscatter title("Test of Conditional Mean"))
est store cps10
graph save cmp_cps10, replace 
graph export cmp_cps10.eps, replace
mat def par_cps10=e(b)
mat def var_cps10=e(V)
outreg2 [cps10] using condd.xml, e(N ll) se excel replace 

***Testing the validity of the conditioning set
*Impute in CPS, do K-S test and CK test versus true variable for 2010 CPS, compare true and imputed cdf
cddens `condvar' [aw=wgtp], model(`model') par(par_cps10) ks(fs_amnt) ck(fs_amnt,`iter') cdf(fs_amnt) draws(`draw') legend(label(1 "Imputed CPS 2010") label(2 "Reported CPS 2010")) title("CDF of Food Stamp Amounts")
graph save cdf_cps10, replace 
graph export cdf_cps10.eps, replace
*store results
mat def ks=(nullmat(ks)\(e(ks_D_1),e(ks_p_1),e(ks_D_2),e(ks_p_2),e(ks_D),e(ks_p),e(ks_p_cor)))
local rn_ks "cps_vs_cpsimp"
matrix colnames ks=distance1 p_value1 distance2 p_value2 distance_max combined_p corrected_p
*add other results (ck test, summary)
mat def sumd=(e(mean),e(Var),e(skewness),e(kurtosis),e(min),e(max),e(p1),e(p5),e(p10),e(p25),e(p50),e(p75),e(p90),e(p95),e(p99))
mat colnames sumd=Mean Variance Skewness Kurtosis Minimum Maximum Pctile1 Pctile5 Pctile10 Pctile25 Pctile50 Pctile75 Pctile90 Pctile95 Pctile99
local rn_sumd "CPS10_Imputation"
mat def ck=(e(ck),e(p_ck))
local rn_ck "CPS10_Imp_vs_CPS1_report"
mat colnames ck=Test_stat p_value

****Run regressions in 2010 CPS comparing true vs. imputed variable
reg fs_amnt `covar' [pw=wgtp], robust
est store true
cdreg fs_amnt_cd `covar' [pw=wgtp], model(`model') par(par_cps10) condvars(fs_amnt_cd `condvar') variance(var_cps10)
est store imputed

*tables
est table true imputed, stats(N r2) se
*export results
outreg2 [true imputed] using reg_cps10.xml, e(N r2) se excel replace


*****Applications
***Prepare Data
*load ACS recipients that fulfill income restriction and FS receipt is not imputed
append using pooled_imp.dta, gen(tmp)
keep if tmp==0 |  (tmp==1 & acs==1 & fs==1 & ffsp==0 & `maxinc' & agep>18)
drop tmp
*create additional variables
*recode np
cap drop np_*
replace np=7 if np>7
qui: tab np, gen(np_)
drop np_1

*higher order terms of variables->creates first order term as well, so var_* includes all terms, var* will include linear term twice
foreach var of varlist `higher' {
	local i=1
	while `i'<=`order' {
		cap drop `var'_`i'
		gen `var'_`i'=`var'^`i'
		local i=`i'+1
	}
}

***get summary statistics (separate for ACS, CPS)
sum `sstat' if acs==0 [aw=wgtp]
sum `sstat' if acs==1 [aw=wgtp]

*mark original observations
local n=_N
gen orig=1

*impute FS in 2010 ACS
cddens `condvar' if acs==1 [aw=wgtp], model(`model') par(par_cps10) tname(fs_amnt_cd) draws(`draw') save(2010_imp.dta, `all')
use 2010_imp.dta, clear
sort orig
replace orig=0 if _n>`n'

*replicate variables I'll use below (only weights and acs indicator)
foreach var of varlist acs wgtp {
	qui: replace `var'=`var'[_n-`n'] if _n>`n'
}
compress
save 2010_imp.dta, replace

***Application 1: Marginal Density
**compare marginal distribution of FS amounts in 2010 ACS using different imputations and reported amounts in CPS
*plot cdf for all variables (fs_amnt_pmm fs_amnt_reg fs_amnt_rules if acs==1, fs_amnt if acs==0)
preserve
if "`n'"=="" {
	qui: total orig
	local n=e(b)
}
*get cumulative dist for each variable & stack them
cumul fs_amnt_cd if acs==1 [aw=wgtp], gen(cdf_cd)
local i=_N
foreach var in hd pmm reg rules {
	cumul fs_amnt_`var' if orig==1 & acs==1 [aw=wgtp], gen(cdf_`var')
	sum fs_amnt_`var' if orig==1 & acs==1, meanonly
	local c=r(N)
	local j=_N+`c'
	qui: set obs `j'
	mata: st_view(a=.,(1,`n'),"fs_amnt_`var' cdf_`var'","touse")
	mata: st_select(c=.,a,a[,1]:!=.)
	mata: st_view(b=.,(`j'-`c'+1,`j'),"fs_amnt_cd cdf_`var'")
	mata: b[,]=c
	mata: a[,2]=J(rows(a),1,.)
}
*add cps
cumul fs_amnt if orig==1 & acs==0 [aw=wgtp], gen(cdf_cps)
sum fs_amnt if orig==1 & acs==0, meanonly
local c=r(N)
local j=_N+`c'
qui: set obs `j'
gen tmp=(acs==0 & orig==1)
mata: st_view(a=.,(1,`n'),"fs_amnt cdf_cps","tmp")
mata: st_select(c=.,a,a[,1]:!=.)
mata: st_view(b=.,(`j'-`c'+1,`j'),"fs_amnt_cd cdf_cps")
mata: b[,]=c
mata: a[,2]=J(rows(a),1,.)
drop tmp

*plot
*Note: see version from 120616 for other ways of plotting this that include all points, but result in bad graphs
qui: replace touse=1 if _n>`i'
*CDF sometimes decreases at vertical parts, line below should prevent that
*sort fs_amnt_cd cdf_cd cdf_hd cdf_pmm cdf_reg cdf_rules cdf_cps
twoway line cdf_cd fs_amnt_cd if mod(_n,2*`draw')==0 & touse==1 & fs_amnt_cd>-1000 & fs_amnt_cd<=15000, sort lp(dash_dot) || line cdf_hd fs_amnt_cd if mod(_n,50)==0 & touse==1 & fs_amnt_cd>-1000 & fs_amnt_cd<=15000, sort lp(shortdash) || line cdf_pmm fs_amnt_cd if mod(_n,50)==0 & touse==1 & fs_amnt_cd>-1000 & fs_amnt_cd<=15000, sort lp(dash) || line cdf_reg fs_amnt_cd if mod(_n,50)==0 & touse==1 & fs_amnt_cd>-1000 & fs_amnt_cd<=15000, sort lp(longdash) || line cdf_rules fs_amnt_cd if mod(_n,50)==0 & touse==1 & fs_amnt_cd>-1000 & fs_amnt_cd<=15000, sort lp(shortdash_dot) || line cdf_cps fs_amnt_cd if touse==1 & fs_amnt_cd>-1000 & fs_amnt_cd<=15000, sort lp(solid) xtitle("Food Stamp Amount") legend(label(1 "Conditional Density") label(2 "Classic Hot Deck") label(3 "PMM") label(4 "Conditional Mean") label(5 "Program Rules") label(6 "CPS report"))
graph save cdf_acs10, replace 
graph export cdf_acs10.eps, replace
*clean up
mata: mata drop a b c
restore

*get results of sum, d for all imputations
sum fs_amnt if orig==1 & acs==0 [aw=wgtp], d
mat def sumd=(nullmat(sumd)\r(mean),r(Var),r(skewness),r(kurtosis),r(min),r(max),r(p1),r(p5),r(p10),r(p25),r(p50),r(p75),r(p90),r(p95),r(p99))
mat colnames sumd=Mean Variance Skewness Kurtosis Minimum Maximum Pctile1 Pctile5 Pctile10 Pctile25 Pctile50 Pctile75 Pctile90 Pctile95 Pctile99
local rn_sumd "`rn_sumd' CPS_Report"

sum fs_amnt_cd if acs==1 [aw=wgtp], d
mat def sumd=(sumd\(r(mean),r(Var),r(skewness),r(kurtosis),r(min),r(max),r(p1),r(p5),r(p10),r(p25),r(p50),r(p75),r(p90),r(p95),r(p99)))
local rn_sumd "`rn_sumd' Cond_Dens_Imp"

foreach var in hd pmm reg rules {
	sum fs_amnt_`var' if orig==1 & acs==1 [aw=wgtp], d
	mat def sumd=(sumd\(r(mean),r(Var),r(skewness),r(kurtosis),r(min),r(max),r(p1),r(p5),r(p10),r(p25),r(p50),r(p75),r(p90),r(p95),r(p99)))
	local rn_sumd "`rn_sumd' `var'_Imp"
}
*export
svmat sumd, names(col)
gen nsumd=""
local j=1
foreach x in `rn_sumd' {
	replace nsumd="`x'" if _n==`j'
	local j=`j'+1
}
local b=rowsof(sumd)
export excel nsumd Mean Variance Skewness Kurtosis Minimum Maximum Pctile1 Pctile5 Pctile10 Pctile25 Pctile50 Pctile75 Pctile90 Pctile95 Pctile99 using 03_analysis.xlsx if _n<=`b', sheetreplace firstrow(var) sh("sumd")
drop nsumd Mean Variance Skewness Kurtosis Minimum Maximum Pctile1 Pctile5 Pctile10 Pctile25 Pctile50 Pctile75 Pctile90 Pctile95 Pctile99

*test distributions of imputations (in ACS) vs. distribution in CPS
*test CPS vs. CD imputation
preserve
gen tmp="fs_amnt_cd" if acs==1
sum fs_amnt if orig==1 & acs==0, meanonly
local c=r(N)
local j=_N+`c'
qui: set obs `j'
gen tmp2=(orig==1 & acs==0)
mata: st_view(a=.,(1,`i'),"fs_amnt wgtp","tmp2")
mata: st_select(c=.,a,a[,1]:!=.)
mata: st_view(b=.,(`i'+1,`j'),"fs_amnt_cd wgtp")
mata: b[,]=c
replace tmp="CPS Response" if _n>`i'
ksmirnov2 fs_amnt_cd [aw=wgtp], by(tmp) inflate(1 `draw')
*store results
mat def ks=(nullmat(ks)\(r(D_1),r(p_1),r(D_2),r(p_2),r(D),r(p),r(p_cor)))
local rn_ks "`rn_ks' fs_amnt_cd_vs_cps_response"
*test CPS vs. other imptations
replace tmp="" if _n<=`i'
foreach var in fs_amnt_hd fs_amnt_pmm fs_amnt_reg fs_amnt_rules {
	replace tmp="`var'" if _n<=`i' & acs==1 & orig==1
	mata: st_view(b=.,(`i'+1,`j'),"`var' wgtp")
	mata: b[,]=c
	ksmirnov2 `var' [aw=wgtp], by(tmp)
	*store results
	mat def ks=(nullmat(ks)\(r(D_1),r(p_1),r(D_2),r(p_2),r(D),r(p),r(p_cor)))
	local rn_ks "`rn_ks' `var'_vs_cps_response"
}
*clean up
mata: mata drop a b c
restore

*test distributions from CD imp vs. all other immputations
gen tmp="Conditional Density Imputation"
foreach var in fs_amnt_hd fs_amnt_pmm fs_amnt_reg fs_amnt_rules {
	preserve
	sum `var' if orig==1 & acs==1, meanonly
	local c=r(N)
	local j=_N+`c'
	qui: set obs `j'
	mata: st_view(a=.,(1,`n'),"`var' wgtp","touse")
	mata: st_select(c=.,a,a[,1]:!=.)
	mata: st_view(b=.,(`j'-`c'+1,`j'),"fs_amnt_cd wgtp")
	mata: b[,]=c
	replace tmp="`var'" if _n>`i'
	replace touse=1 if _n>`i'
	ksmirnov2 fs_amnt_cd [aw=wgtp] if touse==1, by(tmp) inflate(`draw' 1)
	*store results
	mat def ks=(nullmat(ks)\(r(D_1),r(p_1),r(D_2),r(p_2),r(D),r(p),r(p_cor)))
	local rn_ks "`rn_ks' acs_cd_vs_`var'"
	*clean up
	mata: mata drop a b c
	restore
}
drop tmp

*export results of ks and ck tests
matrix colnames ks=distance1 p_value1 distance2 p_value2 distance_max combined_p corrected_p
svmat ks, names(col)
gen nks=""
local b=rowsof(ks)
local j=1
tokenize "`rn_ks'"
while `j'<=`b' {
	replace nks="``j''" if _n==`j'
	local j=`j'+1
}
export excel nks distance1 p_value1 distance2 p_value2 distance_max combined_p corrected_p using 03_analysis.xlsx if _n<=`b', firstrow(var) sheetreplace sh("ks-tests_fs")
*clean up
drop nks distance1 p_value1 distance2 p_value2 distance_max combined_p corrected_p

mat colnames ck=Test_stat p_value
svmat ck, names(col)
gen nck=""
local b=rowsof(ck)
local j=1
tokenize "`rn_ck'"
while `j'<=`b' {
	replace nck="``j''" if _n==`j'
	local j=`j'+1
}
export excel nck Test_stat p_value using 03_analysis.xlsx if _n<=`b', firstrow(var) sheetreplace sh("ck-tests_fs")
*clean up
drop nck Test_stat p_value
keep if orig==1

***Application 2: FS Regressions
*regress FS amounts on HH characteristics for different imp in 2010 ACS and report in 2010 CPS
foreach x in hd pmm reg rules {
	reg fs_amnt_`x' `covar' if acs==1 [aw=wgtp], robust
	est store `x'
}
reg fs_amnt `covar' if acs==0 [aw=wgtp], robust
est store reported
cdreg fs_amnt_imp `covar' if acs==1 [pw=wgtp], model(`model') par(par_cps10) condvars(fs_amnt_imp `condvar') variance(var_cps10)
est store imputed

*tables
est table reported imputed hd pmm reg rules, stats(N r2) se

*export results
outreg2 [reported imputed hd pmm reg rules] using reg_acs10.xml, e(N r2) se excel replace


***Application 3: regress income on (imputed) hours worked in ACS
*Sample already restricted in 01_: wkhp!=. & pernp>0 & agep>=18 & schg==. & occp<9800 & cow>=1 & cow<=7 & fwkhp==0 & wkw==1 (work hours not missing, positive earnings, 18 and older, not in school or army, not working without pay or not at all, work hours not imputed, worked 50 weeks or more no GQ)
use a3_acs_extract.dta, clear

*split sample into source and missing data (50/50)
gen imp=(runiform()>0.5)

**other imputations 
*conditional mean imputation
gen wkhp_reg=wkhp if imp==0
reg wkhp `condvar2' [pw=pwgtp] if imp==0
predict tmp if imp==1
replace wkhp_reg=tmp if imp==1
drop tmp

***predictive means match
*MI setup
mi set wide
gen wkhp_pmm=wkhp if imp==0
mi register imputed wkhp_pmm
*do imputations
mi impute pmm wkhp_pmm `pmmvar2' [pw=pwgtp],add(1)
replace wkhp_pmm=_1_wkhp_pmm if imp==1
*get rid of mi settings
mi unset
drop wkhp_pmm_1_ mi_miss

***classic hot deck
mi set wide
gen wkhp_hd=wkhp if imp==0
mi register imputed wkhp_hd
*do imputations
mi impute pmm wkhp_hd [pw=pwgtp],add(1) by(`hdcells2')
replace wkhp_hd=_1_wkhp_hd if imp==1
*get rid of mi settings
mi unset
drop wkhp_hd_1_ mi_miss

****get summary statistics
sum `sstat2' [aw=pwgtp]

***estimate and test conditional density of hours worked-> check if this works
*generate points to evaluate cmp
pctile p_acs10=wkhp if imp==0 [pw=pwgtp], nq(51)
*run conddens on observed sample
conddens wkhp `condvar2' if imp==0 [pw=pwgtp], model(`model2') cmp(at(p_acs10) noscatter title("Test of Conditional Mean"))
est store wkh
graph save cmp_wkh, replace 
graph export cmp_wkh.eps, replace
mat def par_wkh=e(b)
mat def var_wkh=e(V)
outreg2 [wkh] using condd_wkh.xml, e(N ll) se excel replace 

*get programs to perfectly predict non-imp (mass point at obs=true with probability 1 if non-imp and probability 0 if imputed)
gen wkhp_obs=wkhp if imp==0
replace wkhp_obs=0 if imp==1
*modify parameter vector to include observed
*parameter vector for mp (large value for slope on wkhp_obs, negative value for intercept-> Pr(true=obs)=1 for obs!=0, =0 for obs=0).Works since wkhp_obs>0
mat def tmp=J(1,colsof(par_wkh)-1,0)
mat def tmp[1,1]=10000
mat def tmp[1,colsof(par_wkh)-1]=-1000
*add equation and parameter names
mat coleq tmp=mp
mat colnames tmp=wkhp_obs `condvar2' _cons

*add slope parameter for wkhp_obs for density
mat def par_wkh2=0
mat coleq par_wkh2=mean
mat colnames par_wkh2=wkhp_obs
*add mp parameters
mat def par_wkh2=(tmp,par_wkh2,par_wkh)

*add rows and columns to variance matrix
mat def var_wkh2=J(colsof(par_wkh2),colsof(par_wkh2),0)
mat def var_wkh2[colsof(par_wkh2)-colsof(var_wkh)+1,colsof(par_wkh2)-colsof(var_wkh)+1]=var_wkh

*tests
cddens wkhp_obs `condvar2' [aw=pwgtp], model(`model2') mass par(par_wkh2) ks(wkhp) cdf(wkhp wkhp_hd wkhp_pmm wkhp_reg) draws(1) legend(label(1 "Conditional Density") label(2 "Reported ACS 2010") label(3 "Classic Hot Deck") label(4 "PMM") label(5 "Conditional Mean")) title("CDF of Hours Worked")
graph save cdf_wkhp, replace 
graph export cdf_wkhp.eps, replace

*run regressions: true ACS, all imputation methods
foreach x in hd pmm reg {
	reg pernp wkhp_`x' `covar2' [aw=pwgtp], robust
	est store `x'
}
*full sample
reg pernp wkhp `covar2' [aw=pwgtp], robust
est store reported
*complete cases
reg pernp wkhp `covar2' [aw=pwgtp] if imp==0, robust
est store compcase
*cond dens imp
cdreg pernp wkhp_cd `covar2' [pw=pwgtp], model(`model2') mass par(par_wkh2) condvars(wkhp_cd wkhp_obs `condvar2') variance(var_wkh2)
est store imputed

*tables
est table reported compcase imputed hd pmm reg, stats(N r2) se

*export results
outreg2 [reported compcase imputed hd pmm reg] using a3_reg.xml, e(N r2) se excel replace

cap log close

